#!/bin/sh
################################################################################
#Purpose: Used to apply custom configuration changes on CentOS/RHEL 6.2/6.4/6.5
################################################################################

echo -e "\n\tCustom Configuration"
echo -e "\t######################\n"

echo "WARNING: This script is intended to be executed only on Master Target servers. "
echo "         If this server is not a Master Target, please do not continue. "
echo
while [ "X$Choice" = "X" ]
do
    echo -n "Are you sure you want to continue? [Y|N] [default N] : "
    read Choice
    if [ -z "$Choice" -o "$Choice" = "N" -o "$Choice" = "n" ]; then
        Choice="N" ; exit 1
    elif [ "$Choice" = "Y" -o "$Choice" = "y" ];then
        Choice="Y" 
    else
        Choice=""
        echo -e "Invalid answer. Please retry.\n"
    fi
done

# Make sure only root can run our script
echo -n "Checking for root privileges ... "
[[ $EUID -ne 0 ]] && echo "FAILED" && echo "This script must be run as root" && exit 1
echo "OK"

# Adding Kernel Parameters
echo -n "Adding Kernel Parameters ... "
sed -i -e "s/kernel.sysrq.*/kernel.sysrq = 1/g" /etc/sysctl.conf
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Disabling SELINUX
echo -n "Disabling SELINUX ... "
sed -i -e "s/^SELINUX=.*/SELINUX=disabled/g" /etc/selinux/config
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Set core file size to unlimited
echo -n "Set core file size to unlimited ... "
sed -i -e '/ulimit -c unlimited/d' /root/.bash_profile
echo "ulimit -c unlimited" >> /root/.bash_profile
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Enable kdump by adding entry in /etc/grub.conf
echo -n "Configuring grub to enable Kdump ... "
grubby --update-kernel=ALL --args="crashkernel=128M@64M"
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Creating default multipath.conf file
echo -n "Configuring multipath to blacklist cciss disk ... "
/sbin/mpathconf --enable --with_multipathd y --user_friendly_names n > /dev/null 2>&1
sed -i -e '/^blacklist[[:space:]]*{/ a devnode "^cciss!c[0-9]d[0-9]*"' /etc/multipath.conf
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Black listing root device and enabled multipath
echo -n "Blacklist root disk and enable multipath... "
cat > /usr/sbin/blacklistrootdev.sh << EOF
BootDevice=`mount | grep "on / " | awk '{print $1}'`
BootDevice=`echo ${BootDevice} | sed -e "s:[sp]*[0-9]*$::"`
WWID=`/lib/udev/scsi_id --whitelisted --device=$BootDevice`
[[ -n "$WWID" ]] && sed -i -e "/^blacklist.*{/ a wwid $WWID" /etc/multipath.conf || exit 1
if [ -f /.blacklistrootdev ] ; then
    sed -i "/^wwid/d" /etc/multipath.conf ; sed -i -e "/^blacklist.*{/ a wwid $WWID" /etc/multipath.conf
    dracut --force --add multipath --include /etc/multipath /etc/multipath
    [[ $? -eq 0 ]] && echo '#####  REBOOT THE MACHINE #####'
fi
EOF
chmod 755 /usr/sbin/blacklistrootdev.sh
[[ -f /etc/multipath.conf ]] && sh /usr/sbin/blacklistrootdev.sh > /var/log/blacklistrootdev.log 2>&1
touch /.blacklistrootdev > /dev/null 2>&1
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Enable Firewall and block only ICMP
echo -n "Configuring firewall rules ... "
echo >/etc/sysconfig/iptables >/dev/null 2>&1
cat >/etc/sysconfig/iptables <<EOF-IPTABLES
# Firewall configuration written by system-config-firewall
# Manual customization of this file is not recommended.
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
-A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
-A INPUT -p icmp --icmp-type echo-request -j ACCEPT
-A INPUT -p icmp --icmp-type echo-reply -j ACCEPT
-A INPUT -p icmp -j DROP
-A INPUT -j ACCEPT
-A INPUT -i eth+ -p udp -m udp --dport 694 -j ACCEPT
-A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT
-A FORWARD -i lo -j ACCEPT
-A FORWARD -i eth+ -j ACCEPT
-A OUTPUT -p icmp --icmp-type echo-request -j ACCEPT
-A OUTPUT -p icmp --icmp-type echo-reply -j ACCEPT
-A OUTPUT -p icmp -j DROP
-A OUTPUT -j ACCEPT
COMMIT
EOF-IPTABLES
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Add the below rules to file /etc/audit/audit.rules file
echo -n "Configuring audit rules ... "
cat >>/etc/audit/audit.rules <<EOF-AUDITRULES
-a always,exit -F arch=b64 -F dir=/home -S unlinkat -S unlink -S link -S linkat
-a always,exit -F arch=b64 -F dir=/usr -S unlinkat -S unlink -S link -S linkat
-a always,exit -F arch=b32 -F dir=/home -S unlinkat -S unlink -S link -S linkat
-a always,exit -F arch=b32 -F dir=/usr -S unlinkat -S unlink -S link -S linkat
EOF-AUDITRULES
# Change the num_logs and max_log_file values in audit.conf
sed -i -e "s/num_logs =.*/num_logs = 15/g; s/max_log_file =.*/max_log_file = 20/g" /etc/audit/auditd.conf
[[ $? -ne 0 ]] && echo "FAILED" || echo "OK"

# Disable unnecessary service
echo  "Disabling unnecessary services ... OK"
for ServiceName in NetworkManager abrt-ccpp abrtd acpid blk-availability cachefilesd certmonger cpuspeed cups edac gpm haldaemon ip6tables irqbalance iscsi iscsid libvirt-guests lvm2-monitor  mdmonitor portreserve restorecond sysstat systemtap trace-cmd uuidd
do
    chkconfig --level 12345 "$ServiceName" off >/dev/null 2>&1
done

# Enable necessary service
echo -e "Enabling necessary services ... OK"
/sbin/chkconfig kdump on  > /dev/null 2>&1
/sbin/chkconfig multipathd on  > /dev/null 2>&1
/sbin/chkconfig iptables on  > /dev/null 2>&1

echo -e "*** NOTE: REBOOT IS REQUIRED FOR CUSTOM CHANGES TO TAKE EFFECT.***\n"
echo -e "\n\t********** COMPLETED ************"
echo -e "\t##################################\n"

