#!/bin/sh

export pkg_dir=/opt/draas_packages
export WGET=/usr/bin/wget
export RPM=/bin/rpm
export MKDIR=/bin/mkdir
export CHKCONFIG=/sbin/chkconfig
export SERVICE=/sbin/service
export SED=/bin/sed
export MULTIPATH_CONF=/etc/multipath.conf
export LVM_CONF=/etc/lvm/lvm.conf
export MULTIPATH=/sbin/multipath
export AWK=/bin/awk
export HEAD=/usr/bin/head
export MKFS_EXT4=/sbin/mkfs.ext4
export RETENTION_DEV=""
export PVCREATE=/sbin/pvcreate
export VGCREATE=/sbin/vgcreate
export LVCREATE=/sbin/lvcreate
export ETCHOSTS=/etc/hosts
export FSTAB_CONF=/etc/fstab
export MOUNT=/bin/mount
export RM=/bin/rm
export VIRT_WHAT=/usr/sbin/virt-what
export RETENTION_MNTPATH=/mnt/retention
export TEE=/usr/bin/tee
export CP=/bin/cp
export STAT=/usr/bin/stat
export UMOUNT=/bin/umount
export LVREMOVE=/sbin/lvremove
export VGREMOVE=/sbin/vgremove
export PVREMOVE=/sbin/pvremove
export LOGDIR=/var/log/ConfigureMT.log
export BASEURL=""
export RELEASE=""
export PKGLIST=""

LOGANDDISP ()
{
    eval "$1 2>> $LOGDIR| tee -a $LOGDIR"
}

LOGONLY ()
{
    echo $1 >> $LOGDIR
}

check_already_configured ()
{
    [[ -f /etc/.appliedmtcustomchanges ]] && LOGANDDISP 'echo -e "Error: Custom configuration changes are already applied. Aborting..."' && exit 1

}

verify_virt_env ()
{
    LOGONLY "Checking virtual environment cmd: $VIRT_WHAT"
    if [ ! -f $VIRT_WHAT ]
    then
        LOGANDDISP 'echo "The Virtual Environment is not hyperv. Aborting..."'
        exit 1 
    fi
    VIRT_ENV=`$VIRT_WHAT`
    if [ "$VIRT_ENV" != "hyperv" ]
    then
        LOGANDDISP 'echo "The Virtual Environment is not hyperv. Aborting..."'
        exit 1
    fi
}

remove_lvm ()
{
    LOGONLY "Running cmd: $LVREMOVE --force /dev/mapper/retention_vg-retention_vol >>$LOGDIR 2>&1"
    $LVREMOVE --force /dev/mapper/retention_vg-retention_vol >>$LOGDIR 2>&1
    LOGONLY "Running cmd: $VGREMOVE --force retention_vg >>$LOGDIR 2>&1"
    $VGREMOVE --force retention_vg >>$LOGDIR 2>&1
}

clean_last_failed_state_66 ()
{
    
    LOGONLY "Running cmd: $RPM -e device-mapper-multipath-0.4.9-80.el6.x86_64 >>$LOGDIR 2>&1"
    $RPM -e device-mapper-multipath-0.4.9-80.el6.x86_64 >>$LOGDIR 2>&1
    LOGONLY "Running cmd: $RPM -e device-mapper-multipath-libs-0.4.9-80.el6.x86_64 >>$LOGDIR 2>&1"
    $RPM -e device-mapper-multipath-libs-0.4.9-80.el6.x86_64 >>$LOGDIR 2>&1
    LOGONLY "Running cmd: $RPM -e lsscsi-0.23-2.el6.x86_64 >>$LOGDIR 2>&1"
    $RPM -e lsscsi-0.23-2.el6.x86_64 >>$LOGDIR 2>&1
    LOGONLY "Running cmd: $RPM -e  kpartx-0.4.9-80.el6.x86_64 --nodeps >>$LOGDIR 2>&1"
    $RPM -e kpartx-0.4.9-80.el6.x86_64 --nodeps >>$LOGDIR 2>&1
}

clean_last_failed_state_common ()
{
    if [ -f /etc/lvm/lvm.conf.bkup ]
    then
        LOGONLY "Running cmd: $CP /etc/lvm/lvm.conf.bkup $LVM_CONF >>$LOGDIR 2>&1"
        $CP /etc/lvm/lvm.conf.bkup $LVM_CONF >>$LOGDIR 2>&1
    fi
    if [ -f /etc/fstab.bkup ]
    then
        LOGONLY "Running cmd: $CP /etc/fstab.bkup $FSTAB_CONF >>$LOGDIR 2>&1"
        $CP /etc/fstab.bkup $FSTAB_CONF >>$LOGDIR 2>&1
    fi
    LOGONLY "Running cmd: $UMOUNT -l /mnt/retention >>$LOGDIR 2>&1"
    $UMOUNT -l /mnt/retention >>$LOGDIR 2>&1
    #remove_lvm
    LOGONLY "Running cmd: $RM -rf /opt/draas_packages/ /etc/.appliedcustomchanges /etc/multipath.conf /.blacklistrootdev /mnt/retention >>$LOGDIR 2>&1"
    $RM -rf /opt/draas_packages/ /etc/.appliedcustomchanges /etc/multipath.conf /.blacklistrootdev /mnt/retention >>$LOGDIR 2>&1
    LOGONLY "End Cleaning last failed state left with this script:"
}

clean_last_failed_state ()
{
    LOGONLY "Start Cleaning last failed state left with this script:"
    if [ "$RELEASE" = "6.6" ]
    then
        clean_last_failed_state_66
    else
        # Remove packages in reverse order in PKGLIST
        i=0
        total=`echo $PKGLIST | awk '{print NF}'`
        while [ $i -lt $total ]
        do
            # Get the ith package name from the end
            pkg="`echo "$PKGLIST" | awk -v j=$i '{print $(NF-j)}'`"
            LOGONLY "Running cmd: $RPM -e $pkg >>$LOGDIR 2>&1"
            rpm -e $pkg >>$LOGDIR 2>&1
            ((i++))
        done
    fi

    clean_last_failed_state_common
}

create_pkg_dir ()
{
    if [ -d $pkg_dir ]
    then
        LOGANDDISP 'echo "Directory $pkg_dir alreday exists. Removing it..."'
        $RM -rf $pkg_dir
    else
        LOGONLY "Creating directory cmd: $MKDIR -p $pkg_dir >>$LOGDIR 2>&1"
    fi

    $MKDIR -p $pkg_dir >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Failed to create directory $pkg_dir, rerun this script to try it again. Aborting..."'
        exit 1
    fi
}

change_pkg_dir ()
{
    LOGONLY "Changed directory to downloaded package directory cmd: cd $pkg_dir"
    cd $pkg_dir 
}

catch_net_err ()
{
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to download the rpm package '$1' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
        exit 1
    fi
}

catch_rpm_err ()
{
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to install the rpm package '$1', rerun this script to try it again. Aborting..."'
        exit 1
    fi
}

retry_pkg_availability ()
{
    LOGONLY "Checking pkg availability cmd: $WGET -q --tries=2 --timeout=60 --spider http://vault.centos.org/6.6/os/x86_64/Packages/$1 >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 --spider http://vault.centos.org/6.6/os/x86_64/Packages/$1 >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to find availability for rpm package '$1' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
        exit 1
    fi
}

download_pkg ()
{
    LOGONLY "Downloading pkg cmd:$WGET http://vault.centos.org/6.6/os/x86_64/Packages/$1 >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 http://vault.centos.org/6.6/os/x86_64/Packages/$1 >>$LOGDIR 2>&1
}

download_dep_pkgs_66 ()
{
    retry_pkg_availability 'device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm'
    download_pkg 'device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm'
    catch_net_err 'device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm'

    retry_pkg_availability 'device-mapper-multipath-0.4.9-80.el6.x86_64.rpm'
    download_pkg 'device-mapper-multipath-0.4.9-80.el6.x86_64.rpm'
    catch_net_err 'device-mapper-multipath-0.4.9-80.el6.x86_64.rpm'

    retry_pkg_availability 'kpartx-0.4.9-80.el6.x86_64.rpm'
    download_pkg 'kpartx-0.4.9-80.el6.x86_64.rpm'
    catch_net_err 'kpartx-0.4.9-80.el6.x86_64.rpm'

    retry_pkg_availability 'lsscsi-0.23-2.el6.x86_64.rpm'
    download_pkg 'lsscsi-0.23-2.el6.x86_64.rpm'
    catch_net_err 'lsscsi-0.23-2.el6.x86_64.rpm'
}

download_dep_pkgs_common ()
{
    retry_pkg_availability 'python-pyasn1-0.0.12a-1.el6.noarch.rpm'
    download_pkg 'python-pyasn1-0.0.12a-1.el6.noarch.rpm'
    catch_net_err 'python-pyasn1-0.0.12a-1.el6.noarch.rpm'

    #pkg_url=http://apt.sw.be/redhat/el5/en/i386/rpmforge/RPMS/python-pyasn1-0.0.12a-1.el5.rf.noarch.rpm
    #LOGONLY "Checking pkg availability cmd:$WGET $pkg_url >>$LOGDIR 2>&1" 
    #$WGET -q --tries=2 --timeout=60 --spider $pkg_url >>$LOGDIR 2>&1
    #if [ $? -ne 0 ]
    #then
    #    LOGANDDISP 'echo "Not able to find availability for rpm package '$pkg_url' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
    #    exit 1
    #fi
    #LOGONLY "downloading pkg cmd:$WGET $pkg_url >>$LOGDIR 2>&1"
    #$WGET -q --tries=2 --timeout=60 $pkg_url >>$LOGDIR 2>&1
    #catch_net_err 'python-pyasn1-0.0.12a-1.el5.rf.noarch.rpm'


    pkg_url=http://olcentgbl.trafficmanager.net/openlogic/6/openlogic/x86_64/RPMS/WALinuxAgent-2.0.8-1.noarch.rpm
    LOGONLY "Checking pkg availability cmd:$WGET $pkg_url >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 --spider $pkg_url >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to find availability for rpm package '$pkg_url' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
        exit 1
    fi
    LOGONLY "downloading pkg cmd:$WGET $pkg_url >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 $pkg_url >>$LOGDIR 2>&1
    catch_net_err 'WALinuxAgent-2.0.8-1.noarch.rpm'


    pkg_url=http://download.microsoft.com/download/0/8/E/08EE288C-B81E-425A-85D1-67EA7155AB7E/LinuxICv35.iso
    LOGONLY "Checking pkg availability cmd:$WGET $pkg_url >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 --spider $pkg_url >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to find availability for rpm package '$pkg_url' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
        exit 1
    fi
    LOGONLY "downloading pkg cmd:$WGET $pkg_url >>$LOGDIR 2>&1"
    $WGET -q --tries=2 --timeout=60 $pkg_url >>$LOGDIR 2>&1
    catch_net_err 'LinuxICv35.iso'
}

download_dep_pkgs ()
{
    if [ "$RELEASE" = "6.6" ]
    then
        download_dep_pkgs_66
    else
        # For Non 6.6 MT
        if [ "$RELEASE" = "" ]
        then
            LOGANDDISP 'echo "Release not set ... this is a bug"'
            exit 1
        fi

        if [ "$BASEURL" = "" ]
        then
            LOGANDDISP 'echo "Base URL not set ... this is a bug"'
            exit 1
        fi

        if [ "$PKGLIST" = "" ]
        then
            LOGANDDISP 'echo "Package list not set ... this is a bug"'
            exit 1
        fi

        for pkg in $PKGLIST
        do
            LOGONLY "Checking pkg availability cmd: $WGET -q --tries=2 --timeout=60 --spider $BASEURL/${pkg}.rpm >>$LOGDIR 2>&1"
            $WGET -q --tries=2 --timeout=60 --spider $BASEURL/${pkg}.rpm >>$LOGDIR 2>&1
            if [ $? -ne 0 ]
            then
                LOGANDDISP 'echo "Not able to find availability for rpm package '$pkg' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
                exit 1
            fi

            LOGONLY "Downloading pkg cmd:$WGET $BASEURL/${pkg}.rpm >>$LOGDIR 2>&1"
            $WGET -q --tries=2 --timeout=60 $BASEURL/${pkg}.rpm >>$LOGDIR 2>&1
            if [ $? -ne 0 ]
            then
                LOGANDDISP 'echo "Not able to download the rpm package '${pkg}.rpm' right now.Please resolve network connectivity with web and rerun this script to try it again. Aborting..."'
                exit 1
            fi
        done
    fi

    # Download packages common across all MT versions
    download_dep_pkgs_common
}

install_dep_pkgs_66 ()
{
    LOGONLY "Installing rpm pkg cmd:$RPM -ivh device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1"
    $RPM -ivh device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1 
    catch_rpm_err 'device-mapper-multipath-libs-0.4.9-80.el6.x86_64.rpm'

    LOGONLY "Installing rpm pkg cmd:$RPM -ivh --force kpartx-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1"
    $RPM -ivh --force kpartx-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1 
    catch_rpm_err 'kpartx-0.4.9-80.el6.x86_64.rpm'

    LOGONLY "Installing rpm pkg cmd:$RPM -ivh device-mapper-multipath-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1"
    $RPM -ivh device-mapper-multipath-0.4.9-80.el6.x86_64.rpm >>$LOGDIR 2>&1 
    catch_rpm_err 'device-mapper-multipath-0.4.9-80.el6.x86_64.rpm'

    LOGONLY "Installing rpm pkg cmd:$RPM -ivh lsscsi-0.23-2.el6.x86_64.rpm >>$LOGDIR 2>&1"
    $RPM -ivh lsscsi-0.23-2.el6.x86_64.rpm >>$LOGDIR 2>&1 
    catch_rpm_err 'lsscsi-0.23-2.el6.x86_64.rpm'
}

install_dep_pkgs ()
{
    if [ "$RELEASE" = "6.6" ]
    then
        install_dep_pkgs_66
    else
        for pkg in $PKGLIST
        do
            if [ ! -f ${pkg}.rpm ]
            then
                LOGANDDISP 'echo "Package ${pkg}.rpm not found"'
                exit 1
            fi

            LOGONLY "Installing rpm pkg cmd:$RPM -ivh ${pkg}.rpm >>$LOGDIR 2>&1"
            rpm -ivh --force ${pkg}.rpm
            catch_rpm_err "${pkg}.rpm"
        done
    fi
}

configure_multiapth ()
{
# Creating default multipath.conf file by enabling multipath
LOGANDDISP 'echo -n "Enabling multipath ... "'
/sbin/mpathconf --enable --with_multipathd y --user_friendly_names n >> $LOGDIR 2>&1
[[ $? -ne 0 ]] && LOGANDDISP 'echo "FAILED"' || LOGANDDISP 'echo "OK"'

# Blacklisting root device
LOGANDDISP 'echo -n "Blacklisting root device ... "'
cat > /usr/sbin/blacklistrootdev.sh << EOF
#!/bin/sh
if [ ! -e /.blacklistrootdev ] ; then
    #black list root device
    BootDevice=\`mount | grep "on / " | awk '{print \$1}'\`
    BootDevice=\`echo \${BootDevice} | sed -e "s:[sp]*[0-9]*$::"\`
    WWID=\`/lib/udev/scsi_id --whitelisted --device=\$BootDevice\`
    [[ -n "\$WWID" ]] && sed -i -e "/^blacklist.*{/ a wwid \$WWID" /etc/multipath.conf || exit 1

    #black list swap device
    SwapDevice=\`mount | grep "on /mnt/resource " | awk '{print \$1}'\`
    SwapDevice=\`echo \${SwapDevice} | sed -e "s:[sp]*[0-9]*$::"\`
    WWID=\`/lib/udev/scsi_id --whitelisted --device=\$SwapDevice\`
    [[ -n "\$WWID" ]] && sed -i -e "/^blacklist.*{/ a wwid \$WWID" /etc/multipath.conf || exit 1

    touch /.blacklistrootdev > /dev/null 2>&1
fi
EOF
chmod 755 /usr/sbin/blacklistrootdev.sh >/dev/null 2>&1
[[ -f /etc/multipath.conf ]] && sh /usr/sbin/blacklistrootdev.sh > /var/log/blacklistrootdev.log 2>&1
[[ $? -ne 0 ]] && LOGANDDISP 'echo "FAILED"' || LOGANDDISP 'echo "OK"'

}


configure_lvm_conf ()
{
    LOGONLY "Doing backup of lvm conf file cmd: $CP $LVM_CONF /etc/lvm/lvm.conf.bkup"
    $CP $LVM_CONF /etc/lvm/lvm.conf.bkup
    LOGONLY "Start editing lvm conf file"
    sed -e '/# filter \= \[ \"a\/\.\*\/\" \]/ s//filter \= \[ "a\|\/dev\/disk\/by\-id\/dm\-name\-'$RETENTION_DEV'\|\"\, \"r\|\.\*\|\" ]/' -i $LVM_CONF >>$LOGDIR 2>&1
    LOGONLY "Ended editing lvm conf file"
}

catch_retention_err ()
{
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to configure retention drive, please verify if it is already configured or rerun this script to try it again. Aborting..."'
        exit 1
    fi
}

configure_retention_disk_onlvm ()
{
    LOGONLY "Reloading multipath device maps cmd: $MULTIPATH -r >>$LOGDIR 2>&1"
    sleep 5
    $MULTIPATH -r >>$LOGDIR 2>&1
    catch_retention_err

    LOGONLY "Retrieving retention drive name cmd: $MULTIPATH -l|$HEAD -n1|$AWK '{print \$1}' >>$LOGDIR 2>&1"
    sleep 5 
    RETENTION_DEV=`$MULTIPATH -l|$HEAD -n1|$AWK '{print $1}'`
    catch_retention_err
    if [ -z $RETENTION_DEV ]
    then
        LOGANDDISP 'echo "Not able to find retention drive, please first configure retention drive manually afterwards rerun this script to try it again. Aborting..."'
        exit 1
    fi

    LOGONLY "Checking availability of retention drive cmd: $STAT /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    $STAT /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to find retention drive /dev/mapper/$RETENTION_DEV, please first configure retention drive manually afterwards rerun this script to try it again. Aborting..."'
    fi

    LOGONLY "Checking usability for retention drive cmd: blkid -o value -s TYPE /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    VERIFYRETDEVICE=`blkid -o value -s TYPE /dev/mapper/$RETENTION_DEV`
    # don't catch_retention_err here becuase command return non zero value when device don't have FS or LVM, which is valid in our case.
    if [ -n $VERIFYRETDEVICE ] && [ "$VERIFYRETDEVICE" != "LVM2_member" ]
    then
        LOGANDDISP 'echo "Retention drive /dev/mapper/$RETENTION_DEV is already in use, please first clean the retention drive manually afterwards rerun this script to try it again. Aborting..."'
        exit 1
    fi
    sleep 5 

    LOGONLY "Wiping physical volume cmd: $PVREMOVE --force /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    $PVREMOVE --force /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1
    sleep 5 

    LOGONLY "Creating pysical volume cmd: $PVCREATE /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    $PVCREATE /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1 
    catch_retention_err
    sleep 5 

    LOGONLY "Creating volume group cmd: $VGCREATE retention_vg /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    $VGCREATE retention_vg /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1 
    catch_retention_err
    sleep 5 

    LOGONLY "Creating logical volume cmd: $LVCREATE -l 100%FREE -n retention_vol retention_vg >>$LOGDIR 2>&1"
    $LVCREATE -l 100%FREE -n retention_vol retention_vg >>$LOGDIR 2>&1 
    catch_retention_err
    sleep 5 

    LOGANDDISP 'echo "Creating file system on retention drive, depending upon size of device it may take long time. Please wait...."'
    $MKFS_EXT4 /dev/mapper/retention_vg-retention_vol >/dev/null 2>&1 
    catch_retention_err
    LOGANDDISP 'echo "Successfully created file system on retention drive.Mounting it..."'
    sleep 5 

    LOGONLY "Creating retention directory cmd: $MKDIR $RETENTION_MNTPATH >>$LOGDIR 2>&1"
    $MKDIR $RETENTION_MNTPATH >>$LOGDIR 2>&1
    catch_retention_err
    sleep 5 

    LOGONLY "Mounting retention drive cmd: $MOUNT -t ext4 /dev/mapper/retention_vg-retention_vol $RETENTION_MNTPATH >>$LOGDIR 2>&1"
    $MOUNT -t ext4 /dev/mapper/retention_vg-retention_vol $RETENTION_MNTPATH >>$LOGDIR 2>&1 
    catch_retention_err

    LOGONLY "Doing backup of $FSTAB_CONF file cmd: cp $FSTAB_CONF /etc/fstab.bkup"
    $CP $FSTAB_CONF /etc/fstab.bkup
    LOGONLY "Adding retention mount point entry inside /etc/fstab"
    echo -e "/dev/mapper/retention_vg-retention_vol $RETENTION_MNTPATH ext4 rw 0 0" >> $FSTAB_CONF
    catch_retention_err
    sleep 5 
}

configure_retention_disk ()
{
    LOGONLY "Reloading multipath device maps cmd: $MULTIPATH -r >>$LOGDIR 2>&1"
    sleep 5
    $MULTIPATH -r >>$LOGDIR 2>&1
    catch_retention_err

    LOGONLY "Retrieving retention drive name cmd: $MULTIPATH -l|$HEAD -n1|$AWK '{print \$1}' >>$LOGDIR 2>&1"
    sleep 5
    RETENTION_DEV=`$MULTIPATH -l|$HEAD -n1|$AWK '{print $1}'`
    catch_retention_err
    if [ -z $RETENTION_DEV ]
    then
        LOGANDDISP 'echo "Not able to find retention drive, please first configure retention drive manually afterwards rerun this script to try it again. Aborting..."'
        exit 1
    fi

    LOGONLY "Checking availability of retention drive cmd: $STAT /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1"
    $STAT /dev/mapper/$RETENTION_DEV >>$LOGDIR 2>&1
    if [ $? -ne 0 ]
    then
        LOGANDDISP 'echo "Not able to find retention drive /dev/mapper/$RETENTION_DEV, please first configure retention drive manually afterwards rerun this script to try it again. Aborting..."'
    fi

    LOGANDDISP 'echo "Creating file system on retention drive, depending upon size of device it may take long time. Please wait...."'
    $MKFS_EXT4 /dev/mapper/$RETENTION_DEV >/dev/null 2>&1
    catch_retention_err
    LOGANDDISP 'echo "Successfully created file system on retention drive. Mounting it..."'
    sleep 5

    LOGONLY "Creating retention directory cmd: $MKDIR $RETENTION_MNTPATH >>$LOGDIR 2>&1"
    $MKDIR $RETENTION_MNTPATH >>$LOGDIR 2>&1
    catch_retention_err
    sleep 5

    LOGONLY "Mounting retention drive cmd: $MOUNT -t ext4 /dev/mapper/$RETENTION_DEV $RETENTION_MNTPATH >>$LOGDIR 2>&1"
    $MOUNT -t ext4 /dev/mapper/$RETENTION_DEV $RETENTION_MNTPATH >>$LOGDIR 2>&1
    catch_retention_err

    LOGONLY "Doing backup of $FSTAB_CONF file cmd: $CP $FSTAB_CONF /etc/fstab.bkup"
    $CP $FSTAB_CONF /etc/fstab.bkup
    LOGONLY "Adding retention mount point entry inside /etc/fstab"
    UUID=`blkid /dev/mapper/$RETENTION_DEV | awk '{print $2}'`
    catch_retention_err
    echo -e "$UUID $RETENTION_MNTPATH ext4 rw 0 0" >> $FSTAB_CONF
    catch_retention_err
    sleep 5
}

configure_etchosts ()
{
    HN=`hostname`
    HNFQDN=`hostname --fqdn`
    if [ $? -ne 0 ]
    then
        LOGONLY "hostname --fqdn command failed. Adding hostname inside $ETCHOSTS file"
        HNFQDN=$HN
    fi 
    HOST_IP=`ifconfig |sed -e "/vmnet/,/^$/ d" |grep 'inet addr:'|grep -v '127.0.0.1' | grep -v '0.0.0.0' | cut -d: -f2 |awk '{ print $1}'`
    if [ -z "$HOST_IP" ]; then
        LOGANDDISP 'echo " Could not find valid IP address. Aborting..."'
        exit 1
    fi
    for ip in ${HOST_IP}
    do
        available=`grep -w $ip $ETCHOSTS`
        if [ -z "$available" ]; then
            LOGONLY "Host name ( $HN ) does not match with IP address $ip in $ETCHOSTS. Adding entry inside $ETCHOSTS file cmd: echo $ip $HNFQDN $HN >>$ETCHOSTS"
            echo $ip $HNFQDN $HN >>$ETCHOSTS
        else
            LOGONLY "Host name ( $HN ) matches with IP address $ip in $ETCHOSTS. Skipping to edit $ETCHOSTS file..."
        fi
    done
}

create_dummy_file ()
{
    # Creating dummy file to restrict multiple execution
    touch /etc/.appliedmtcustomchanges >/dev/null 2>&1
}

config_init ()
{
    RELEASE="`lsb_release  -r | awk '{print $NF}'`"
    if [ "$RELEASE" = "6.5" ]
    then
        BASEURL="http://vault.centos.org/6.5/os/x86_64/Packages/"
        # Maintain the order in which you want to install packages in PKGLIST
        # The reverse order is used for uninstall in clean_last_failed_state()
        PKGLIST="device-mapper-multipath-libs-0.4.9-72.el6.x86_64 kpartx-0.4.9-72.el6.x86_64 device-mapper-multipath-0.4.9-72.el6.x86_64 lsscsi-0.23-2.el6.x86_64"
    fi
}

Auto_Setup_MT ()
{
    config_init
    check_already_configured
    verify_virt_env
    configure_etchosts
    clean_last_failed_state
    create_pkg_dir
    change_pkg_dir
    download_dep_pkgs
    install_dep_pkgs
    configure_multiapth
    #configure_retention_disk_onlvm
    configure_retention_disk
    configure_lvm_conf
    create_dummy_file
}

Auto_Setup_MT

