#! /bin/sh

PATH=/bin:/sbin:/usr/sbin:/usr/bin:$PATH
export PATH
 
# Get the following details during installation
ACTL_KERNEL_VER="SET_ACTL_KERNEL_VER"
S_FILE_SUFFIX="SET_S_FILE_SUFFIX"
SEP="SET_SEP"
KERNEL_VER_SUFFIX="SET_KERNEL_VER_SUFFIX"
DEPLOY_DIR="SET_DEPLOY_DIR"
OS="SET_OS"
CHOSEN_AGENT_CONFIG="SET_CHOSEN_AGENT_CONFIG"
DRIVERS="involflt"

echo >> /var/log/InMage_drivers.log
echo `date` >> /var/log/InMage_drivers.log
echo "----------------------------" >> /var/log/InMage_drivers.log

if echo $OS | grep -q 'RHEL\|SLES' && echo $CHOSEN_AGENT_CONFIG | grep -q host ; then
    for driver in $DRIVERS
    do
        case $driver in
            involflt) driver_dir="kernel/drivers/char" ;;            
        esac

        for suffix in $KERNEL_VER_SUFFIX
        do
            for k_dir in $(eval echo /lib/modules/${ACTL_KERNEL_VER}*${SEP}${suffix}/${driver_dir})
            do
                if [ -d ${k_dir} ]; then
                    echo >> /var/log/InMage_drivers.log
                    echo "Kernel Directory : ${k_dir}" >> /var/log/InMage_drivers.log
                    if [ -f ${k_dir}/${driver}.ko ]; then
                        echo "${driver}.ko already exists in ${k_dir}" >> /var/log/InMage_drivers.log
                        RET_VAL=256
                    else
                        cp ${DEPLOY_DIR}/bin/${driver}.ko.${S_FILE_SUFFIX}*${SEP}${suffix} ${k_dir}/${driver}.ko
                        RET_VAL=$?
                    fi
                    if [ "${RET_VAL}" = "0" ]; then
                        echo "Installed ${driver}.ko successfully under ${k_dir}" >> /var/log/InMage_drivers.log
                        VER_DIR=$(dirname $(dirname $(dirname $k_dir)))
                        K_VER=${VER_DIR##*/}
                        echo "Running depmod for kernel $K_VER ..." >> /var/log/InMage_drivers.log
                        depmod $K_VER

                        if [ $? -eq 0 ]; then
                            echo "depmod successful for above kernel..." >> /var/log/InMage_drivers.log
			    if [ "$driver" = "involflt" ]; then
			        if [ "$OS" = "SLES11-SP3-32" -o "$OS" = "SLES11-SP3-64" ]; then
			            ${DEPLOY_DIR}/bin/inm_mkinitrd_sles.sh install >> /var/log/InMage_drivers.log 2>&1
			            if [ $? -eq 0 ]; then
		                        echo "Initrd images created successfully." >> /var/log/InMage_drivers.log 2>&1
			            else
			                echo "Initrd images creation failed." >> /var/log/InMage_drivers.log 2>&1
			            fi
			        fi
			    fi
                        else
                            echo "depmod did not succeed for kernel $K_VER ..." >> /var/log/InMage_drivers.log
                            echo "This may affect loading of drivers when you boot this machine using the kernel $K_VER!" >> /var/log/InMage_drivers.log
                        fi
                    elif [ "${RET_VAL}" = "256" ]; then
                        :
                    else
                        echo "Could not install ${driver}.ko under ${k_dir}" >> /var/log/InMage_drivers.log
                    fi
                fi
            done
        done
    done
elif [ "${OS}" = "OL5-32" -o "${OS}" = "OL5-64" ]; then
    echo "OS is OL5-32 or OL5-64. Copying involflt.ko to new kernels which are installed after agent installation." >> /var/log/InMage_drivers.log
    # Copy involflt.ko to RHEL standard kernels matching 2.6.18*.el5, 2.6.18*.el5xen and 2.6.18*.el5PAE
    # Copy involflt.ko to uek kernels matching 2.6.32-100*.el5, 2.6.32-100*.el5uek, 2.6.32-200*.el5uek, 2.6.32-300*.el5uek and 2.6.32-400*.el5uek
    # Copy involflt.ko to uek2 kernels matching 2.6.39-100*.el5uek, 2.6.39-200*.el5uek, 2.6.39-300*.el5uek and 2.6.39-400*.el5uek

    for kernel in 2.6.18-274.el5 2.6.18-274.el5xen 2.6.18-274.el5PAE \
		  2.6.32-100.26.2.el5 2.6.32-100.35.1.el5uek 2.6.32-200.13.1.el5uek 2.6.32-300.10.1.el5uek 2.6.32-400.26.1.el5uek \
		  2.6.39-100.5.1.el5uek 2.6.39-200.24.1.el5uek 2.6.39-300.17.1.el5uek 2.6.39-400.17.1.el5uek
    do
	# Based on kernel version decide kernel and suffix
	if [ "${kernel}" = "2.6.18-274.el5" ]; then
	    KER=2.6.18
	    SUF=el5
	elif [ "${kernel}" = "2.6.18-274.el5xen" ]; then
	    KER=2.6.18
	    SUF=el5xen
	elif [ "${kernel}" = "2.6.18-274.el5PAE" ]; then
	    KER=2.6.18
	    SUF=el5PAE
	elif [ "${kernel}" = "2.6.32-100.26.2.el5" ]; then
	    KER=2.6.32-100
	    SUF=el5
	elif [ "${kernel}" = "2.6.32-100.35.1.el5uek" ]; then
	    KER=2.6.32-100
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.32-200.13.1.el5uek" ]; then
	    KER=2.6.32-200
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.32-300.10.1.el5uek" ]; then
	    KER=2.6.32-300
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.32-400.26.1.el5uek" ]; then
	    KER=2.6.32-400
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.39-100.5.1.el5uek" ]; then
	    KER=2.6.39-100
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.39-200.24.1.el5uek" ]; then
	    KER=2.6.39-200
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.39-300.17.1.el5uek" ]; then
	    KER=2.6.39-300
	    SUF=el5uek
	elif [ "${kernel}" = "2.6.39-400.17.1.el5uek" ]; then
	    KER=2.6.39-400
	    SUF=el5uek
	fi

        # Find all available kernel directories matching kernel version and suffix
        for k_dir in `find /lib/modules/${KER}*${SUF} -maxdepth 0 -type d 2>/dev/null`
        do
            NEW_KERNEL=0

            # Copy involflt.ko to each kernel directory
            if [ -f ${k_dir}/kernel/drivers/char/involflt.ko ]; then
                echo "${k_dir}/kernel/drivers/char/involflt.ko already exists" >> /var/log/InMage_drivers.log
            else
                # Copy involflt.ko to each kernel directory
                if [ -d ${k_dir}/kernel/drivers/char ]; then
                    echo "Copying ${DEPLOY_DIR}/bin/involflt.ko.${kernel} to ${k_dir}/kernel/drivers/char/involflt.ko" >> /var/log/InMage_drivers.log
                    cp -f ${DEPLOY_DIR}/bin/involflt.ko.${kernel} ${k_dir}/kernel/drivers/char/involflt.ko
                    if [ $? -eq 0 ]; then
                        echo "Installed involflt.ko successfully under ${k_dir}" >> /var/log/InMage_drivers.log
                        NEW_KERNEL=1
                    fi
                else
                    echo "Could not find ${k_dir}/kernel/drivers/char directory" >> /var/log/InMage_drivers.log
                fi
            fi

            if [ "${NEW_KERNEL}" -eq 1 ]; then
                # Run depmod if it is new kernel
                echo "Running depmod for kernel $k_dir ..." >> /var/log/InMage_drivers.log
                echo "" >> /var/log/InMage_drivers.log
                depmod $k_dir
            fi
        done
    done
elif [ "${OS}" = "OL6-32" -o "${OS}" = "OL6-64" ]; then
    echo "OS is OL6-32 or OL6-64. Copying involflt.ko to new kernels which are installed after agent installation." >> /var/log/InMage_drivers.log
    # Copy involflt.ko to RHEL standard kernels matching 2.6.32-*el6.i686 2.6.32-*el6.x86_64
    # Copy involflt.ko to uek kernels matching 2.6.32-100*el6uek.i686, 2.6.32-200*el6uek.i686, 2.6.32-300*el6uek.i686 and 2.6.32-400.*el6uek.i686
    # Copy involflt.ko to uek2 kernels matching 2.6.39-100*el6uek.i686, 2.6.39-200*el6uek.i686, 2.6.39-300*el6uek.i686 and 2.6.39-400.*el6uek.i686
    # Copy involflt.ko to uek kernels matching 2.6.32-100*el6.x86_64, 2.6.32-100*el6uek.x86_64, 2.6.32-200*el6uek.x86_64, 2.6.32-300*el6uek.x86_64 and 2.6.32-400*el6uek.x86_64
    # Copy involflt.ko to uek2 kernels matching 2.6.39-100*el6uek.x86_64, 2.6.39-200*el6uek.x86_64, 2.6.39-300*el6uek.x86_64 and 2.6.39-400*el6uek.x86_64

    for kernel in 2.6.32-131.0.15.el6.i686 2.6.32-131.0.15.el6.x86_64 \
                  2.6.32-100.34.1.el6uek.i686 2.6.32-200.16.1.el6uek.i686 2.6.32-300.3.1.el6uek.i686 2.6.32-400.26.1.el6uek.i686 \
                  2.6.39-100.5.1.el6uek.i686 2.6.39-200.24.1.el6uek.i686 2.6.39-300.17.1.el6uek.i686 2.6.39-400.17.1.el6uek.i686 \
                  2.6.32-100.28.5.el6.x86_64 2.6.32-100.34.1.el6uek.x86_64 2.6.32-200.16.1.el6uek.x86_64 2.6.32-300.3.1.el6uek.x86_64 2.6.32-400.26.1.el6uek.x86_64 \
                  2.6.39-100.5.1.el6uek.x86_64 2.6.39-200.24.1.el6uek.x86_64 2.6.39-300.17.1.el6uek.x86_64 2.6.39-400.17.1.el6uek.x86_64 3.8.13-16.2.1.el6uek.x86_64
    do
        # Based on kernel version decide kernel and suffix
        if [ "${kernel}" = "2.6.32-131.0.15.el6.i686" ]; then
            KER=2.6.32
            SUF=el6.i686
        elif [ "${kernel}" = "2.6.32-131.0.15.el6.x86_64" ]; then
            KER=2.6.32
            SUF=el6.x86_64
        elif [ "${kernel}" = "2.6.32-100.34.1.el6uek.i686" ]; then
            KER=2.6.32-100
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.32-200.16.1.el6uek.i686" ]; then
            KER=2.6.32-200
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.32-300.3.1.el6uek.i686" ]; then
            KER=2.6.32-300
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.32-400.26.1.el6uek.i686" ]; then
            KER=2.6.32-400
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.39-100.5.1.el6uek.i686" ]; then
            KER=2.6.39-100
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.39-200.24.1.el6uek.i686" ]; then
            KER=2.6.39-200
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.39-300.17.1.el6uek.i686" ]; then
            KER=2.6.39-300
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.39-400.17.1.el6uek.i686" ]; then
            KER=2.6.39-400
            SUF=el6uek.i686
        elif [ "${kernel}" = "2.6.32-100.28.5.el6.x86_64" ]; then
            KER=2.6.32-100
            SUF=el6.x86_64
        elif [ "${kernel}" = "2.6.32-100.34.1.el6uek.x86_64" ]; then
            KER=2.6.32-100
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.32-200.16.1.el6uek.x86_64" ]; then
            KER=2.6.32-200
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.32-300.3.1.el6uek.x86_64" ]; then
            KER=2.6.32-300
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.32-400.26.1.el6uek.x86_64" ]; then
            KER=2.6.32-400
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.39-100.5.1.el6uek.x86_64" ]; then
            KER=2.6.39-100
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.39-200.24.1.el6uek.x86_64" ]; then
            KER=2.6.39-200
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.39-300.17.1.el6uek.x86_64" ]; then
            KER=2.6.39-300
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "2.6.39-400.17.1.el6uek.x86_64" ]; then
            KER=2.6.39-400
            SUF=el6uek.x86_64
        elif [ "${kernel}" = "3.8.13-16.2.1.el6uek.x86_64" ]; then
            KER=3.8.13
            SUF=el6uek.x86_64
        fi

        # Find all available kernel directories matching kernel version and suffix
        for k_dir in `find /lib/modules/${KER}*${SUF} -maxdepth 0 -type d 2>/dev/null`
        do
            NEW_KERNEL=0

            # Copy involflt.ko to each kernel directory
            if [ -f ${k_dir}/kernel/drivers/char/involflt.ko ]; then
                echo "${k_dir}/kernel/drivers/char/involflt.ko already exists" >> /var/log/InMage_drivers.log
            else
                # Copy involflt.ko to each kernel directory
                if [ -d ${k_dir}/kernel/drivers/char ]; then
                    echo "Copying ${DEPLOY_DIR}/bin/involflt.ko.${kernel} to ${k_dir}/kernel/drivers/char/involflt.ko" >> /var/log/InMage_drivers.log
                    cp -f ${DEPLOY_DIR}/bin/involflt.ko.${kernel} ${k_dir}/kernel/drivers/char/involflt.ko
                    if [ $? -eq 0 ]; then
                        echo "Installed involflt.ko successfully under ${k_dir}" >> /var/log/InMage_drivers.log
                        NEW_KERNEL=1
                    fi
                else
                    echo "Could not find ${k_dir}/kernel/drivers/char directory" >> /var/log/InMage_drivers.log
                fi
            fi

            if [ "${NEW_KERNEL}" -eq 1 ]; then
                # Run depmod if it is new kernel
                echo "Running depmod for kernel $k_dir ..." >> /var/log/InMage_drivers.log
                echo "" >> /var/log/InMage_drivers.log
                depmod $k_dir
            fi
        done
    done
else
    echo "Either installation is Fabric or OS is not a RHEL/SLES/OL platform" >> /var/log/InMage_drivers.log
fi
