#!/bin/bash
#
# Copyright (c) 2012 InMage
# This file contains proprietary and confidential information and
# remains the unpublished property of InMage. Use, disclosure,
# or reproduction is prohibited except as permitted by express
# written license aggreement with InMage.
#
# File       : vCon service script
#
# Revision - history 
#
#
#
# Description:
# vCon service script to accomplish tasks for vCon workflow
#

PATH=/bin:/sbin:/usr/sbin:/usr/bin:$PATH
export PATH
RETVAL=1
vcon_action_file="/etc/vxagent/vcon/p2v_actions"
DATE_STRING=`date +'%b_%d_%H_%M_%S'`
CDROM_DEVICE="/dev/cdrom"
DVDROM="/dev/dvd"
DVDROM1="/dev/dvd1"
DVDROM2="/dev/sr0"
VMWARETOOLS_DEVICE=""
TEMP_DIR="/tmp/install_vmware_tools_vcon_${DATE_STRING}"
VMWARE_MNT_PATH="/tmp/install_vmware_tools_vcon_${DATE_STRING}/mount-path"
VMWARE_INSTALL_SCRIPT=""
VMWARE_UNINSTALL_SCRIPT="vmware-uninstall-tools.pl"
VMWARE_INSTALL_SCRIPT_OPTIONS="--default"

workflow=`grep workflow $vcon_action_file | awk -F= '{ print $2 }'`
workflow=`echo $workflow | sed -e 's: ::g'`
vmwaretools=`grep install_vmware_tools $vcon_action_file | awk -F= '{ print $2 }'`
initrd=`grep create_initrd_images $vcon_action_file | awk -F= '{ print $2 }'`
vmwaretools_uninstall=`grep uninstall_vmware_tools $vcon_action_file | awk -F= '{ print $2 }'`
vmwaretools=`echo $vmwaretools | sed -e 's: ::g'`
vmwaretools_uninstall=`echo $vmwaretools_uninstall | sed -e 's: ::g'`
_NEW_GUID_=`grep new_guid $vcon_action_file | awk -F= '{ print $2 }'`
OS=""

function install_vmware_tools_vcon
{
    retval=1
    is_rpm_file=0
    is_targz_file=0
    install_filename=""
    # Mount cdrom
    mkdir -p $TEMP_DIR
    if [ $? -ne 0 ]; then
        echo "Failed to create directory ${TEMP_DIR}"
        return $retval
    fi
    mkdir -p ${VMWARE_MNT_PATH}
    mount -t iso9660 -o ro ${VMWARETOOLS_DEVICE} ${VMWARE_MNT_PATH} 2>&1
    if [ $? -eq 0 ]; then
        echo "Mount ${VMWARETOOLS_DEVICE} to ${VMWARE_MNT_PATH} succeeded"
    else
        echo "Mount ${VMWARETOOLS_DEVICE} to ${VMWARE_MNT_PATH} failed"
        return $retval
    fi
    
    ls -l ${VMWARE_MNT_PATH} |  grep -i "VMwareTools"| grep ".rpm$"
    if [ $? -eq 0 ]; then
        is_rpm_file=1
    fi
    ls -l ${VMWARE_MNT_PATH} |  grep -i "VMwareTools"| grep ".tar.gz$"
    if [ $? -eq 0 ]; then
        is_targz_file=1
    fi
    
    if [ $is_targz_file -eq 1 ]; then   
        install_filename=`ls ${VMWARE_MNT_PATH}/VMwareTools*tar.gz`
        cp $install_filename ${TEMP_DIR}
        install_filename=`ls ${TEMP_DIR}/VMwareTools*tar.gz`
        #untar it.
        tar zxf "${install_filename}" -C "${TEMP_DIR}"  2>&1 > /dev/null
        if [ 0 -ne $? ]; then
            echo "Failed to untar file ${install_filename} under path ${TEMP_DIR}."
            umount ${VMWARE_MNT_PATH} 2>&1
            return $retval
        fi
        VMWARE_INSTALL_SCRIPT="${TEMP_DIR}/vmware-tools-distrib/vmware-install.pl"
    else
        if [ $is_rpm_file -eq 1 ]; then   
            install_filename=`ls ${VMWARE_MNT_PATH}/VMwareTools*rpm`
            # install rpm
            rpm -Uvh $install_filename 2>&1 /dev/null
            if [ 0 -ne $? ]; then
                echo "Failed to install rpm ${install_filename} under path ${TEMP_DIR}."
                umount ${VMWARE_MNT_PATH} 2>&1
                return $retval
            fi
            VMWARE_INSTALL_SCRIPT="vmware-config-tools.pl" 
        fi
    fi
    # Install VMWare tools
    ${VMWARE_INSTALL_SCRIPT} ${VMWARE_INSTALL_SCRIPT_OPTIONS} 2>&1
    if [ 0 -ne $? ]; then
        echo  "Failed to install VMware tools."
        umount ${VMWARE_MNT_PATH} 2>&1
        return $retval
    fi
    #umount
    umount ${VMWARE_MNT_PATH} 2>&1
    if [ 0 -eq $? ]; then
        rm -rf ${TEMP_DIR}
    fi    

    # Sucess now set return value
    retval=0

    return $retval
}

function UpdateHostID
{
    #TFS ID 1269459
    vx_dir=""
    if [ -f /usr/local/.vx_version ]; then
        vx_dir=`grep INSTALLATION_DIR /usr/local/.vx_version | awk -F'=' '{print $2 }'`
    else
        if [ -d /usr/local/ASR/Vx ]; then
            vx_dir=/usr/local/ASR/Vx
        else
            echo "InMage Scout product is not installed on the system. Exiting."
            return 1
        fi
    fi

    #Update HostId value in drScout.conf file for VX agent.
    currentHostIdValue=`cat ${vx_dir}/etc/drscout.conf | grep "HostId"`
    currentHostIdValue=$(printf "%s\n" "${currentHostIdValue}" | sed 's/[][\.*^$(){}?+|/]/\\&/g')
    echo "VX: Old HostId Value : ${currentHostIdValue}, New HostId : ${_NEW_GUID_}."
    sed -i "s/${currentHostIdValue}/HostId=${_NEW_GUID_}/" "${vx_dir}/etc/drscout.conf"
    #restart vxagent. service name vxagent.
    #service vxagent restart

    #update HostID value in Config.ini for FX agent.
    fx_dir=""
        if [ -f /usr/local/.fx_version ]; then
            fx_dir=`grep INSTALLATION_DIR /usr/local/.fx_version | awk -F'=' '{print $2 }'`
        else
            if [ -d /usr/local/ASR/Fx ]; then
                fx_dir=/usr/local/ASR/Fx
            else
                echo "InMage Scout product is not installed on the system. Exiting."
                return 1
            fi
        fi
    
    currentHostIdValue=`cat ${fx_dir}/config.ini | grep "HostId"`
    currentHostIdValue=$(printf "%s\n" "${currentHostIdValue}" | sed 's/[][\.*^$(){}?+|/]/\\&/g')
    echo "FX: Old HostId Value : ${currentHostIdValue}, New HostId : ${_NEW_GUID_}."
    sed -i "s/${currentHostIdValue}/HostId = ${_NEW_GUID_}/" "${fx_dir}/config.ini"
    #restart FX agent. service name svagent.
    service svagent restart

    #Should we also restart appSevice?
    return 0
}

function findostype
{
	vx_version_file=`ls -l /usr/local/.vx_version | awk -F'-> ' '{ print $2 }'`

	os_built_for=`grep OS_BUILT_FOR $vx_version_file | awk -F= '{ print $2 }'`

	case $os_built_for in
    	OL*)
             OS=OEL
   	    ;;
   	    RHEL*)
             OS=RHEL
        ;;
    	SUSE*)
            OS=SUSE
        ;; 
    	SLES*)
            OS=SUSE
    	;;
    	DEBIAN*)
            OS=DEBIAN
        ;;
    	UBUNTU-14*)
            OS=UBUNTU14
    	;;
	esac
 } 
 
# Main entry point 

# Create the initrd images to incorporate the protection of Full Disk that
# root filesystem. Calling the script inm_mkinitrd_sles.sh will do this job.
#
# To triiger this action, the action "create_mkinitrd_images=1" should be
# present in the p2v_actions file.

if [ "$initrd" = "1" ] && [ -f /etc/SuSE-release ] &&  grep -q 'VERSION = 11' /etc/SuSE-release && grep -q 'PATCHLEVEL = 3' /etc/SuSE-release; then
    /etc/vxagent/bin/inm_mkinitrd_sles.sh install
    if [ $? -eq 0 ]; then
        echo "Initrd images created successfully"
        sed -i "s:create_initrd_images*=*1:create_initrd_images=0:" ${vcon_action_file}
    else
        echo "Initrd image creation failed"
    fi
fi

case "${workflow}" in

    p2v)
        # Find the OS for which the agents built for.
        findostype
        
        # For UBUNTU-14 do not install/uninstall vmware tools. 
        # Note: For UBUNTU-14 the open-vm-tools are recomended.
        if [ "$OS" = "UBUNTU14" ]; then
             echo "For ubuntu14, installing/uninstalling vmware tools will not be done by this workflow."
			 
             echo "Reseting the install/uninstall options in p2v_actions file."
             sed -i "s:uninstall_vmware_tools*=*1:uninstall_vmware_tools=0:" ${vcon_action_file} 
             sed -i "s:install_vmware_tools*=*1:install_vmware_tools=0:" ${vcon_action_file}
			 
        # Un-Install VMWare tool if uninstall_vmware_tools=1 in p2v_actions file
        elif [ "1" = "$vmwaretools_uninstall" ]; then
            echo "uninstalling VMware tools."
                ${VMWARE_UNINSTALL_SCRIPT}  2>&1
            if [ 0 -ne $? ]; then
                 echo  "Failed to uninstall VMware tools."
                     RETVAL=1
            else
                         sed -i "s:uninstall_vmware_tools*=*1:uninstall_vmware_tools=0:" ${vcon_action_file}
            fi
        # Install VMWare tool if install_vmware_tools=1 in p2v_actions file
        elif [ "1" = "$vmwaretools" ]; then
            if [ -f /usr/bin/vmware-config-tools.pl ]; then
                echo "VMware Tools are already installed."
            fi
       
            # install vmware tools on recovered machine here
            if [ ! -b ${CDROM_DEVICE} ]; then
                echo "Device ${CDROM_DEVICE} is not a block device"
                if [ ! -b ${DVDROM} ]; then
                    echo "Device ${DVDROM} is not a block device"
                    if [ ! -b ${DVDROM1} ]; then
                        echo "Device ${DVDROM1} is not a block device"
                        if [ ! -b ${DVDROM2} ]; then
                            echo "Device ${DVDROM2} is not a block device"
                        else
                            VMWARETOOLS_DEVICE="${DVDROM2}"
                        fi
                    else
                        VMWARETOOLS_DEVICE="${DVDROM1}"
                    fi
                else
                    VMWARETOOLS_DEVICE="${DVDROM}"
                fi
            else
                VMWARETOOLS_DEVICE="${CDROM_DEVICE}"
            fi

            install_vmware_tools_vcon
            if [ $? -ne 0 ]; then
                VMWARETOOLS_DEVICE=""
                for temp_device in `ls /dev/hd*`
                do
                   ide_device=`echo $temp_device | sed -e "s:/dev/::"`
                   ide_device_removable=`cat /sys/block/$ide_device/removable`
                   if [ "${ide_device_removable}" = "1" ]; then
                       VMWARETOOLS_DEVICE=$temp_device
                   fi
                done
                if [ ! -z "${VMWARETOOLS_DEVICE}" ]; then
                    install_vmware_tools_vcon
                    if [ $? -eq 0 ]; then
                        echo "VMWare Tools installation succeeded"
                        sed -i "s:install_vmware_tools*=*1:install_vmware_tools=0:" ${vcon_action_file}
                    else
                        echo "VMWare Tools installation failed"
                    fi
                fi
            else
                echo "VMWare Tools installation succeeded"
                sed -i "s:install_vmware_tools*=*1:install_vmware_tools=0:" ${vcon_action_file}
            fi
        fi

        # Reset HostId with give new hostId from p2v_actions file
        if [ ! -z ${_NEW_GUID_} ]; then
            echo "Found new host GUID value. Going to update HostId value in drScout.conf and Config.ini"
            UpdateHostID
            if [ $? -ne 0 ]; then
                echo "failed to update host id."
            else
                sed -i "s:new_guid*=*${_NEW_GUID_}:new_guid=:" ${vcon_action_file}
            fi
        fi
        RETVAL=0
        ;;

    v2v)
        RETVAL=0
        ;;

    v2p)
    #we have to uninstall VMware tools.
        RETVAL=0
    echo "Uninstall VMware Tools = $vmwaretools_uninstall"
    if [ 1 -eq $vmwaretools_uninstall ]; then
        echo "uninstalling VMware tools."
            ${VMWARE_UNINSTALL_SCRIPT}  2>&1
        if [ 0 -ne $? ]; then
             echo  "Failed to uninstall VMware tools."
                 RETVAL=1
        else
                     sed -i "s:uninstall_vmware_tools*=*1:uninstall_vmware_tools=0:" ${vcon_action_file}
        fi
    fi
        ;;

    *)
        echo "Invalid workflow:$workflow found"
        exit $RETVAL
        ;;
esac

