#!/bin/sh

# FUNCTION : Carries out a command and logs output into an uninstall log
DO()
{

  eval "$1 2>&1 | tee -a $UNINST_LOG"

}

# FUNCTION : Echoes a command into an uninstall log
LOG()
{

  echo "$1" >> $UNINST_LOG

}

# FUNCTION : Print message and quit
SEE_LOG()
{

 DO 'echo && echo "Check the log file $UNINST_LOG for detailed diagnostic messages during uninstallation ..."'
 exit $2

}

Usage() {
    echo
    echo "Usage: $0 [ -Y ] [ -L <Absolute path of the log file (default:/var/log/ua_uninstall.log)> ] [ -h ]"
    echo && exit 1
}

Uninstall_FX()
{

  DO 'echo "Uninstalling the FX Agent ..."'
  CURRENT_INSTALLATION_DIR=`grep INSTALLATION_DIR /usr/local/${FX_INST_VERSION_FILE} | cut -d'=' -f2-`
  cd $CURRENT_INSTALLATION_DIR
  ./uninstall -Y -L $UNINST_LOG 

}

Uninstall_VX()
{

  DO 'echo "Uninstalling the VX agent ..."'
  CURRENT_INSTALLATION_DIR=`grep INSTALLATION_DIR /usr/local/${INST_VERSION_FILE} | cut -d'=' -f2-`
  cd $CURRENT_INSTALLATION_DIR/bin
  ./uninstall -Y -L $UNINST_LOG 

}

StopServices() {

    FX_INSTALL_DIR=`grep INSTALLATION_DIR /usr/local/.fx_version | cut -d'=' -f2-`
    VX_INSTALL_DIR=`grep INSTALLATION_DIR /usr/local/.vx_version | cut -d'=' -f2-`

    # Stopping Vx agent before uninstalling agent
    DO 'echo'
    DO 'echo "Stopping VX agent service..."'    
    touch /usr/local/.norespawn_vxagent >/dev/null 2>&1
    ${VX_INSTALL_DIR}/bin/stop uninstall >> ${UNINST_LOG} 2>&1
    if [ $? -eq 2 ]; then
        DO 'echo'
        DO 'echo "Could not stop Vx agent service. Please re-try after sometime. Aborting..."'
        rm -f /usr/local/.norespawn_vxagent >/dev/null 2>&1
        return 1
    fi

    # Stopping Fx agent before uninstalling agent
    DO 'echo'
    DO 'echo "Stopping Fx agent service..."'
    touch /usr/local/.norespawn_svagent >/dev/null 2>&1
    ${FX_INSTALL_DIR}/stop >> ${UNINST_LOG} 2>&1
    if [ $? -eq 2 ]; then
        DO 'echo'
        DO 'echo "Could not stop Fx agent service. Please re-try after sometime. Aborting..."'
        rm -f /usr/local/.norespawn_svagent >/dev/null 2>&1
        return 1
    fi
    return 0
}

Check_FX_Presence()
{

  FX_INST_VERSION_FILE=.fx_version
  if [ -e /usr/local/${FX_INST_VERSION_FILE} ]; then
	FX_AGENT_FOUND=FX
  fi

}

Check_VX_Presence()
{

  INST_VERSION_FILE=.vx_version
  if [ -e /usr/local/${INST_VERSION_FILE} ]; then
  	VX_AGENT_FOUND=VX
  fi

}

#######################
# MAIN ENTRY POINT
#######################

# Prompt the user to confirm whether to proceed to uninstall the agent really
#Parse command line options if given
if echo $1 | grep -q '^-' ; then
    while getopts :L:hY OPT
    do
        case $OPT in
            L)  LOG_FILE="$OPTARG"
                if expr "$LOG_FILE" : '^-.' > /dev/null; then
                    echo "Option -$OPT requires an argument." && Usage
                fi
            ;;
            Y)  SILENT_UNINSTALL="yes" ;;
            h|-*|*) Usage ;;
        esac
    done
elif [ "$1" = "" ]; then
    SILENT_UNINSTALL="no"
else
    Usage
fi

if  [ -z "$SILENT_UNINSTALL" -o "$SILENT_UNINSTALL" = "no" ] ; then
    # If no option is specified, then the default is interactive    
    echo
    agree=""
    while [ "X$agree" = "X" ]; do
        echo -n "Do you really want to uninstall the Unified agent? (Y/N) [default N] : "
        read ans
        [ -z "$ans" ] && ans="N"
        case $ans in
            [Yy]) agree=Yes ; SILENT_UNINSTALL="no" ;;
            [Nn]) echo "Bye..." && exit 1 ;;
            *) echo -e "Error: Invalid input.\n"
            continue
            ;;
        esac
    done
fi

if [ -z "$LOG_FILE" ]; then 
    if [ ! -d /var/log ]; then 
       mkdir -p /var/log >/dev/null 2>&1
    fi
    UNINST_LOG="/var/log/ua_uninstall.log"
elif [ "`echo $LOG_FILE | cut -c1`" != "/" ]; then
    DO 'echo'
    DO 'echo "Error: Unintallation logfile must be an absolute path (i.e. beginning with a \"/\")."'
    Usage
else
    LOG_FILE_DIR=`dirname "$LOG_FILE"`
    if [ ! -d "$LOG_FILE_DIR" ]; then
        mkdir -p "$LOG_FILE_DIR" >/dev/null 2>&1
    fi
    UNINST_LOG="$LOG_FILE"
fi

LOG "UA UNINSTALLATION"
LOG "********************"
LOG "-----------------------------------"
LOG "`date`"
LOG "-----------------------------------"

Check_FX_Presence

Check_VX_Presence

if [ -z "$FX_AGENT_FOUND" -a -z "$VX_AGENT_FOUND" ]; then
    DO 'echo "Either FX or VX is not installed in the setup ..."'
    DO 'echo "Cannot proceed with uninstallation ..."'
    SEE_LOG 1
fi

touch /usr/local/unified_uninstall

if [ "$FX_AGENT_FOUND" = "FX" -a "$VX_AGENT_FOUND" = "VX" ]; then
	# Stopping FX/VX Agent Services
	# Aborting un-installation if it's failed to stop the services
	StopServices
	[ $? -ne 0 ] && SEE_LOG 2

	Uninstall_VX
	Uninstall_FX

elif [ "$FX_AGENT_FOUND" = "FX" ]; then

	Uninstall_FX

elif [ "$VX_AGENT_FOUND" = "VX" ]; then

	Uninstall_VX

fi

# Removing fingerprint/certificate directories & other files
rm -rf /usr/local/InMage/certs /usr/local/InMage/fingerprints /usr/local/InMage/private >/dev/null 2>&1
rm -f /usr/local/unified_uninstall /usr/local/drscout.conf.lck  /usr/local/.norespawn_* >/dev/null 2>&1
rm -f /usr/local/InMage/license.txt /etc/.appliedmtcustomchanges >/dev/null 2>&1

LOG "END OF UA UNINSTALLATION"
LOG "************************"
